import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.net.*;
import java.util.*;

/**
 * Shell's methods
 * @author Ernestas Kardzys (ernestaslt@gmail.com)
 * @version 0.1
 */
public class BasicShell {
	private static Socket socket = new Socket();
	private final static String CMDPROMPT = ">";
	private final static String HELLOMSG = "Welcome to JavaServer v. 0.1!\r\n";
	private final static String HELPMESSAGE = "Type \"help\" for help.\r\n\r\n"; 
	private final static String CMDHELP = "List of all commands:\r\n";
	private final static String HELP = "help -- commands list;\r\n";
	private final static String DATE = "date -- today's date;\r\n";
	private final static String PING = "ping -- ping host;\r\n";
	private final static String ABOUT = "about -- information about program;\r\n";
	private final static String INFO = "info -- information about the server;\r\n";
	private final static String EXIT = "exit -- exit from program.\r\n"; 
	private final static String UNKNOWNCMD1 = "Undefined command: \"";
	private final static String UNKNOWNCMD2 = "\". Try \"help\".\r\n";
	
	/**
	 * 
	 * @param socket - socket number
	 */
	public BasicShell(Socket socket) {
		BasicShell.socket = socket;
	}
	
	/**
	 * Sends a message over the network.
	 * @param message - message to be sent
	 */
	private void sendMessage(String message) {
		try {
			  BufferedWriter wr = new BufferedWriter(new OutputStreamWriter(socket.getOutputStream()));
			  wr.write(message);
			  wr.flush();
			} 
			catch (IOException e) {
				System.out.println("ERROR: sendMessage()!");
				System.err.println(e.getMessage());
				e.printStackTrace();
			}
	}
	
	/**
	 * Prints hello message for the user
	 */
	public void helloMessage() {
		try {
			BufferedWriter wr = new BufferedWriter(new OutputStreamWriter(socket.getOutputStream()));
			wr.write(HELLOMSG);
			wr.write("Today's date is: " + new Date() + "\r\n");
			wr.write(HELPMESSAGE);
			wr.write(CMDPROMPT);
			wr.flush();
		}
		catch (IOException e) {
			System.err.println(e.getMessage());
			e.printStackTrace();
		}
	}
	
	/**
	 * Processes commands entered by users.
	 * @param command Command to process
	 */
	public void processCommand(String command) {
		// Print help
		if (command.equals("help"))	{
			sendMessage(CMDHELP + HELP + DATE + PING + ABOUT + INFO + EXIT);
		}
		// Print current date
		else if (command.equals("date")) {
			sendMessage(new Date().toString() + "\r\n");
		}
		// Ping host (server pings itself!)
		else if (command.equals("ping")) {
			try {
				InetAddress address = InetAddress.getByName("localhost");
				boolean reachable = address.isReachable(10000);
				sendMessage("Is host reachable? " + reachable + "\r\n"); 
			}
			catch (Exception e) {
				e.printStackTrace();
			}
		}
		// Information about program
		else if (command.equals("about")) {
			sendMessage("JavaServer - I'm learning JAVA, so I decited to write a simple program, " 
					    + "which does something useful :)\r\n");
		}
		// Information about server
		else if (command.equals("info")) {
			String message = new String();
			message = "Address: " + socket.getLocalAddress().toString() + "; Port: "
					  + socket.getLocalPort() + "\r\n";
			sendMessage(message);
		}
		else sendMessage(UNKNOWNCMD1 + command + UNKNOWNCMD2);
		sendMessage(CMDPROMPT);
	}
}
